/*
 * wysmod - wydal.c
 * 	version 0.0.0
 * 
 * Copyright (C) 2005..2010  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2006, 2007  Marek Piwonski <marek@microster.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/serial.h>
#include <tmos/jiffies.h>
#include <tmos/console.h>
#include <tmos/watchdog.h>

#include "ar2clp.h"
#include "display.h"

static uint8_t seqnum = 0xff;
static uint8_t old_keys = 0;

static char wydal_init(void) REENTRANT
{
	__code static uint8_t cmd[] = { 0xff, 0x0f, 0x18,
		CONFIG_WYSMOD_WYDAL_DISPLAY_ID, 0x00, 20, 2, 6,
		0 };
	uint8_t ret;

	if (ar2c_resync())
		return 1;
	ser_tx(cmd, sizeof(cmd));
	if (ser_rxd_t(&ret, 1, MS_JIFFIES(500)) != 1)
		return 1;
	return ret;
}

static void send_keepalive(void)
{
	ser_txc(0x58);
	ser_txc(seqnum);
}

struct fb_msg {
	uint16_t magic;
	uint8_t seqnum;
	uint8_t fb[40];
	uint8_t crc;
};

static void check_keys(void) REENTRANT
{
	uint8_t tmp;

	tmp = KEYS_PORT;
	if (tmp != old_keys) {
		ser_txc(0x85);
		ser_txc(tmp);
		old_keys = tmp;
	}
}

static void wydal_loop(void) REENTRANT
{
	jiffies_t keepalive, host_keepalive;
	jiffies_t t;
	char i = 0;
	struct fb_msg msg;

	keepalive = host_keepalive = get_jiffies();
	for (;;) {
		watchdog_refresh();
		t = get_jiffies();

		/* Co sekunde wysylamy informacje, ze zyjemy */
		if (t - keepalive >= S_JIFFIES(1)) {
			send_keepalive();
			keepalive = t;
		}

		/*
		 * Rejestator moze robic maksymalnie 5 sekundowe przerwy,
		 * jezeli robi dluzsze to przyjmujemy, ze komunikacja
		 * zostala zerwana. Ble lelelel
		 */
		if (t - host_keepalive >= S_JIFFIES(5))
			break;

		if (ser_rx_poll()) {
			((uint8_t *) &msg)[i++] = ser_rxc();

			if (i == 1) {
				if (((uint8_t *) &msg)[0] != 0x11)
					i = 0;
			} else if (i == 2) {
				switch (((uint8_t *) &msg)[1]) {
					case 0x26: /* ramka z obrazem */
						host_keepalive = get_jiffies();
						break;
					case 0xa5: /* keep alive */
						host_keepalive = get_jiffies();
						i = 0;
						break;
					default: /* nieznana wiadomosc */
						i = 0;
						break;
				}
			} else if (i >= sizeof(struct fb_msg)) {
				uint8_t crc;
				char j;

				host_keepalive = get_jiffies();

				crc = msg.crc ^ msg.seqnum;
				for (j = 39; j >= 0; j--)
					crc ^= msg.fb[j];

				if (!crc) {
					c_put(msg.fb);
					c_flush();
					seqnum = msg.seqnum;
				}

				i = 0;
			}
		}
		check_keys();
	}
}

void wydal(void)
{
	for (;;) {
		init_screen();
		if (wydal_init()) {
			sleep(1);
			continue;
		}
		wydal_loop();
	}
}
