/*
 * tmos - reboot.c
 *
 * Copyright (C) 2012  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/reboot.h>

/*
 * By default 8051 starts code execution from address 0. For restart we
 * do the same, we disable interrupts and jump to address 0.
 */
static void generic_restart(void) __reentrant
{
	u16 addr;
	void (*prog)(void);

	local_irq_disable();
	addr = 0x0000;
	prog = (void * __force)(void __code *__force) addr;
	prog();
}

#ifdef CONFIG_ADUC841_HAS_BOOTLOADER
/*
 * ADUC841 supports for read-only bootloader at 0xe000. During restart
 * it's a good idea to check if bootloader exists and jump to bootloader
 * instead of address 0.
 */
static void mach_restart(void)
{
	u16 addr;
	void (*prog)(void);

	addr = 0xe000;
	if (*((u8 __code *) addr) == 0xff)
		return;

	local_irq_disable();
	prog = (void * __force)(void __code *__force) addr;
	prog();
}
#define mach_restart mach_restart
#endif

#ifndef mach_restart
#define mach_restart()	do { } while (0)
#endif

void machine_restart(void) __reentrant
{
	mach_restart();
	generic_restart();
}
