/*
 * tmos - port.c
 * Copyright (C) 2009..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <tmos/kernel.h>
#include <tmos/port.h>
#include <tmos/string.h>
#include <tmos/sched.h>
#include <tmos/errno.h>

/**
 * port_read_timeout - read data from port with maximal interbyte timeout
 * @port: port
 * @buf: buffer to read
 * @len: number of bytes to read
 * @timeout: timeout in jiffies
 */
msg_ssize_t port_read_timeout(port_t port, char __kernel *buf, msg_size_t len,
		jiffies_t timeout) __reentrant
{
	msg_ssize_t readed = 0;
	jiffies_t t0;

	t0 = get_jiffies();
	while (readed < (msg_ssize_t) len) {
		msg_ssize_t ret;
		unsigned char events;
		
		events = __port_poll(port);
		if (events & POLLIN) {
			ret = port_read(port, buf, len - readed);
		} else {
			ret = -EAGAIN;
		}
		if (ret < 0) {
			if (ret == -EAGAIN) {
				if (time_after(get_jiffies(), t0 + timeout)) {
					if (readed == 0)
						readed = -EAGAIN;
					break;
				}
				schedule();
				continue;
			}
			if (readed == 0)
				readed = ret;
			break;
		}
		readed += ret;
		buf += ret;
		if (!ret)
			break;
		t0 = get_jiffies();
	}
	return readed;
}
