/*
 * tmos - jsleep.c
 * 
 * Copyright (C) 2005..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/config.h>
#include <tmos/jiffies.h>
#include <tmos/sched.h>

/**
 * jsleep - sleep
 * @t: the number of jiffies to sleep
 *
 * This function sleeps at least @t jiffies.
 */
void jsleep(jiffies_t t) REENTRANT
{
#if CONFIG_MULTITASKING
	jiffies_t timeout;

	t += get_jiffies();
	do {
		timeout = t - get_jiffies();

		if (timeout >= MAX_SCHEDULE_TIMEOUT)
			timeout = MAX_SCHEDULE_TIMEOUT;
		schedule_timeout(timeout);
	} while (!time_after_eq(get_jiffies(), t));
#else
	t += get_jiffies();
	do {
		schedule();
	} while (!time_after_eq(get_jiffies(), t));
#endif
}
