/*
 * tmos - string.c
 * 
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/string.h>

int tmos_strcmp(const char *asrc, const char *adst) __reentrant
{
	register int ret = 0;

	while (!(ret = *(unsigned char *) asrc - *(unsigned char *)adst)
			&& *adst)
		++asrc, ++adst;
	return ret;
}

#ifndef memset_kernel
void memset_kernel(void __kernel *_dst, unsigned char _c, unsigned char _n)
	__reentrant
{
	unsigned char __kernel *dst = _dst;
	unsigned char c = _c;
	unsigned char n = _n;

	if (n > 0) {
		do {
			*dst++ = c;
		} while (--n > 0);
	}
}
#endif

#ifdef CONFIG_HIGHMEM
#ifndef memset_highmem
void memset_highmem(void __highmem *_dst, unsigned char _c, unsigned char _n)
	__reentrant
{
	unsigned char __highmem *dst = _dst;
	unsigned char c = _c;
	unsigned char n = _n;

#if CONFIG_OPTIMIZE_SPEED
	if (n >= 8) {
		do {
			*dst++ = c;
			*dst++ = c;
			*dst++ = c;
			*dst++ = c;
			*dst++ = c;
			*dst++ = c;
			*dst++ = c;
			*dst++ = c;
			n -= 8;
		} while (n >= 8);
	}

	if (n >= 4) {
		*dst++ = c;
		*dst++ = c;
		*dst++ = c;
		*dst++ = c;
		n -= 4;
	}

	if (n >= 2) {
		*dst++ = c;
		*dst++ = c;
		n -= 2;
	}

	if (n > 0)
		*dst++ = c;
#else
	if (n > 0) {
		do {
			*dst++ = c;
		} while (--n > 0);
	}
#endif
}
#endif
#endif
